//Licence:   LGPL
//Copyright: Visti Andresen

#include "FontGl.h"
#include <iostream>

using namespace std;



FontGl::FontGl()
{
  for (int i = 0; i != 256; i++){
    charWidth_[i] = -1;
  }
}

FontGl::~FontGl()
{
  unloadFontGfx();
}

void FontGl::prepareDrawing()
{
	glViewport(0,0, displayWidth_,displayHeight_);
  
	glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
	gluOrtho2D(0,
                screenWidth_-1,
                screenHeight_-1,
                0);


  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
	glDisable(GL_CULL_FACE);
  
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glDisable(GL_DEPTH_TEST);
  glDisable(GL_FOG);
  glDisable(GL_LIGHTING);
}

void FontGl::loadFontGfx(const string &fileName, const string &fontDefinition)
{
  unloadFontGfx();
  SFontLoader loader;
  loader.setAlphaLoadingType(SFontLoader::GREY_COLOUR);
  loader.setFontDefinition(fontDefinition);
  loader.loadFont(fileName);
  
  fontCellHeight_ = loader.getFontHeight();
  fontCellWidth_ = loader.getFontWidth();
  charHeight_ = loader.getCharacterHeight();
  //cout << "dhgkfsghdfkgsdhdkfg"<< endl;

  for (unsigned int i = 0; i != 256; i++){
    if (loader.getCharacterWidth((char)i) > 0){
      charWidth_[i] = (float)loader.getCharacterWidth((char)i);
      //cout << (int)loader.getDataForCharacter((char)i) << "hdjsa"<< endl;
      GLuint index;  
      glGenTextures(1, &index);
      glBindTexture(GL_TEXTURE_2D, index);
      gluBuild2DMipmaps(GL_TEXTURE_2D, 4, loader.getFontWidth(),
        loader.getFontHeight(), GL_RGBA, GL_UNSIGNED_BYTE,
        loader.getDataForCharacter((char)i));
      images_[i] = index;
    }
  }
}

void FontGl::setScreenSize(float virtualWidth, float virtualHeight,
                           int displayWidth, int displayHeight)
{
  screenWidth_ = virtualWidth;
  screenHeight_ = virtualHeight;
  displayWidth_ = displayWidth;
  displayHeight_ = displayHeight;
}

void FontGl::printChar(const char data, float x, float y, float zoom)
{
	float charWidth = charWidth_[(unsigned char)data]*zoom;
  float charHeight = charHeight_*zoom;
	float texWidth = charWidth/fontCellWidth_/zoom;
	float texHeight= charHeight_/fontCellHeight_;

  glColor3f(1,1,1);
	glBindTexture(GL_TEXTURE_2D, images_[(unsigned char)data]);
	glBegin (GL_QUADS);                              
	
	glTexCoord2f(0.0f, 0.0f);
	glVertex3f(x,y, 0);
	
	glTexCoord2f(texWidth, 0.0f);
	glVertex3f(x+charWidth, y, 0);
	
	glTexCoord2f(texWidth, texHeight);
	glVertex3f(x+charWidth, y+charHeight, 0);
	
	glTexCoord2f(0, texHeight);
	glVertex3f(x, y+charHeight, 0);

	glTexCoord2f(0.0f, 0.0f);
	glVertex3f(x ,y, 0);

	glEnd();
}

void FontGl::printString(const string &value, float x, float y, float zoom)
{
  string myValue = value;
	while(myValue.size()){
		if (myValue[0]!=' '){
			printChar(myValue[0], x, y, zoom);
		}
		else{
      myValue[0] = '-';
    }
		x+= charWidth_[(unsigned char)myValue[0]]*zoom;
		myValue.erase(0,1);
	}
  
}

float FontGl::getWidth(const string &data, float zoom)
{
  float charWidth = 0;
	unsigned char c;
	for (unsigned int i=0; i<data.size(); i++){
		c = data[i];
		if (c == ' ') c='-';
		charWidth += charWidth_[c]*zoom;
	}
	return charWidth;
}

float FontGl::getHeight(float zoom)
{
  return charHeight_*zoom;
}

void FontGl::unloadFontGfx()
{
  for (int i = 0; i != 256; i++){
    if (charWidth_[i] > 0){
      glDeleteTextures(1, &images_[i]);
    }
    charWidth_[i] = -1;
  }
}
