

#ifndef MOD_FONTGL_H
#define MOD_FONTGL_H

//Licence:   LGPL
//Copyright: Visti Andresen

#include "Font.h"
#include "SFontLoader.h"

/**A font for rendering on a OpenGL display.*/ 
class FontGl : public Font
{
public:
  FontGl();
  ~FontGl();
  void prepareDrawing();
  /**Sets display size hints for the font.

  the virtualWidth and virtualHeight defines the range of coordinates
  regardless of actual display size, coordinates and font size will be
  scaled to fit the actual display size.

  displayWidth and displayHeight are the actual display size.
  */
  void setScreenSize(float virtualWidth, float virtualHeight,
                                       int displayWidth, int displayHeight);

  /**Loads a SFont file.

   fileName is the name (may include path) of the SFont.

   fontDefinition is a string containing the letters in the SFont.*/
  void loadFontGfx(const string &fileName, const string &fontDefinition=
"!\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~");
  void printChar(const char data, float x, float y, float zoom);
  void printString(const string &value, float x, float y, float zoom);
  float getWidth(const string &data, float zoom);
  float getHeight(float zoom);
  /**Frees the OpenGL texture memory used by the font, call this before reloading the font.*/
  void unloadFontGfx();

private:
  GLuint images_[256];
  float charWidth_[256];
  float charHeight_;
  float fontCellWidth_;
  float fontCellHeight_;
  float screenWidth_;
  float screenHeight_;
  int displayWidth_;
  int displayHeight_;
};


#endif
