//Licence:   LGPL
//Copyright: Visti Andresen

#include "Key.h"

Key::Key(const string &label, const string &name, Font *font,
         SDLKey defaultValue)
{
	font_  = font;
	name_ = name;
	label_ = label;
	grabbingKey_ = false;
  state_ = false;
  press_ = false;
	thisKey_ = defaultValue;
}

Key::~Key()
{
}

void Key::load(ifstream &file)
{
	if (findLoadPosition(name_, file))
	{
    long value;
		file >> value;
    thisKey_ = (SDLKey)value;
    //cout << label_ << " value " << thisKey_ << endl;
	}
}

void Key::save(ofstream &file)
{
	file << name_ << "=" << (long)thisKey_ << endl;
}

void Key::draw(float x, float y, float zoom)
{
	font_->printString(getStringRepresentation(), x, y, zoom);
}

float Key::getWidth(float zoom)
{
	return font_->getWidth(getStringRepresentation(), zoom);
}
	
float Key::getHeight(float zoom)
{
	return font_->getHeight(zoom);
}
	
string Key::getStringRepresentation()
{
	string keyText="seeing this is a bug";
	if (grabbingKey_){
		keyText = "press new key";
	}
	else {
		char buffer[2];
		buffer[0] = (char)thisKey_;
		buffer[1] = 0;
		if (thisKey_ < 255 && font_->getWidth(buffer, 1)!=-1){
			//font contains char
			keyText = buffer;
		}
		else {
			switch (thisKey_){
				case SDLK_UP:
					keyText = "arrow up";
					break;
				case SDLK_DOWN:
					keyText = "arrow down";
					break;
				case SDLK_LEFT:
					keyText = "arrow left";
					break;
				case SDLK_RIGHT:
					keyText = "arrow right";
					break;
				case SDLK_LCTRL:
					keyText = "left control";
					break;
				case SDLK_RCTRL:
					keyText = "right control";
					break;
				case SDLK_LSHIFT:
					keyText = "left shift";
					break;
				case SDLK_RSHIFT:
					keyText = "right shift";
					break;
				case SDLK_LALT:
					keyText = "left alt";
					break;
				case SDLK_RALT:
					keyText = "right alt";
					break;
				case SDLK_F1:
					keyText = "f1";
					break;
				case SDLK_F2:
					keyText = "f2";
					break;
				case SDLK_F3:
					keyText = "f3";
					break;
				case SDLK_F4:
					keyText = "f4";
					break;
				case SDLK_F5:
					keyText = "f5";
					break;
				case SDLK_F6:
					keyText = "f6";
					break;
				case SDLK_F7:
					keyText = "f7";
					break;
				case SDLK_F8:
					keyText = "f8";
					break;
				case SDLK_F9:
					keyText = "f9";
					break;
				case SDLK_F10:
					keyText = "f10";
					break;
				case SDLK_F11:
					keyText = "f11";
					break;
				case SDLK_F12:
					keyText = "f12";
					break;
				case '\t':
					keyText = "tab";
					break;
				case '\r':
					keyText = "return";
					break;
				case '\b':
					keyText = "backspace";
					break;
				case SDLK_DELETE:
					keyText = "delete";
					break;
				case SDLK_KP0:
					keyText = "keypad 0";
					break;
				case SDLK_KP1:
					keyText = "keypad 1";
					break;
				case SDLK_KP2:
					keyText = "keypad 2";
					break;
				case SDLK_KP3:
					keyText = "keypad 3";
					break;
				case SDLK_KP4:
					keyText = "keypad 4";
					break;
				case SDLK_KP5:
					keyText = "keypad 5";
					break;
				case SDLK_KP6:
					keyText = "keypad 6";
					break;
				case SDLK_KP7:
					keyText = "keypad 7";
					break;
				case SDLK_KP8:
					keyText = "keypad 8";
					break;
				case SDLK_KP9:
					keyText = "keypad 9";
					break;
				case SDLK_KP_PERIOD:
					keyText = "keypad period";
					break;
				case SDLK_KP_DIVIDE:
					keyText = "keypad devide";
					break;
				case SDLK_KP_MULTIPLY:
					keyText = "keypad multiply";
					break;
				case SDLK_KP_MINUS:
					keyText = "keypad minus";
					break;
				case SDLK_KP_PLUS:
					keyText = "keypad plus";
					break;
				case SDLK_KP_ENTER:
					keyText = "keypad enter";
					break;
				case SDLK_KP_EQUALS:
					keyText = "keypad equals";
					break;
				case SDLK_INSERT:
					keyText = "insert";
					break;
				case SDLK_HOME:
					keyText = "home";
					break;
				case SDLK_END:
					keyText = "end";
					break;
				case SDLK_PAGEUP:
					keyText = "page up";
					break;
				case SDLK_PAGEDOWN:
					keyText = "page down";
					break;
				case SDLK_NUMLOCK:
					keyText = "numlock";
					break;
				case SDLK_CAPSLOCK:
					keyText = "capslock";
					break;
				case SDLK_SCROLLOCK:
					keyText = "scrollock";
					break;
				case SDLK_RMETA:
					keyText = "right meta";
					break;
				case SDLK_LMETA:
					keyText = "left meta";
					break;
				case SDLK_RSUPER:
					keyText = "right windows";
					break;
				case SDLK_LSUPER:
					keyText = "left windows";
					break;
				case SDLK_MODE:
					keyText = "mode shift";
					break;
				case SDLK_HELP:
					keyText = "help";
					break;
				case SDLK_PRINT:
					keyText = "print screen";
					break;
				case SDLK_SYSREQ:
					keyText = "sys req.";
					break;
				case SDLK_BREAK:
					keyText = "break";
					break;
				case SDLK_MENU:
					keyText = "menu";
					break;
				case SDLK_POWER:
					keyText = "power";
					break;

				default:
					keyText = "no graphics";
					break;
			}
		}
	}
	string result = label_ + ": '"+keyText+"'";
	return result;
}

string &Key::getName()
{
	return name_;
}

bool Key::isPressed()
{
	return state_;
}

bool Key::keyPress()
{
  bool result = press_;
  press_ = false;
  return result;
}

void Key::handleGameKeyboardEvent(SDL_Event *event)
{
  if (event->type == SDL_KEYDOWN && thisKey_ == event->key.keysym.sym){
    state_=true;
    press_=true;
  }
  if (event->type == SDL_KEYUP && thisKey_ == event->key.keysym.sym){
    state_=false;
    press_=false;
  }
}

bool Key::handleKeyEvent(SDL_Event *event)
{
	if (event->type != SDL_KEYDOWN) return false;	
	if (grabbingKey_){
		thisKey_ = event->key.keysym.sym;
		grabbingKey_ = false;
		return true;
	}
	else {
		if (event->key.keysym.sym == SDLK_RETURN){
			grabbingKey_ = true;
			return true;
		}
		else {
			return false;
		}
	}
}
