

#ifndef MOD_KEY_H
#define MOD_KEY_H

//Licence:   LGPL
//Copyright: Visti Andresen

#include "MenuItem.h"
#include "Font.h"

/**A class for configuring keyboard control and handling the keyboard input.*/
class Key : public MenuItem
{
public:
  /**Constructor
    
    lable is the name the user can see on screen.

    name is the name of the TextEntry in the config file.

    font is the Font to use when rendering.

    defaultValue is the SDLKey to use when no configration is loaded.
  */
	Key(const string &label, const string &name, Font *font, 
      SDLKey defaultValue = SDLK_SPACE);
	~Key();

	void load(ifstream &file);
	
	void save(ofstream &file);
	void draw(float x, float y, float zoom);
	Type getType(){return CONTROL;}
	float getWidth(float zoom);
	float getHeight(float zoom);	
	string &getName();

  /**Returns true if the key is currently pressed.*/
	bool isPressed();
  /**Returns true if the key has been pressed since the last time keyPress
  has been called.*/
  bool keyPress();
		
  /**Call this in your event loop with the events to make the key register 
  the keyboard events (Otherwise isPressed and keyPress won't work)*/
	void handleGameKeyboardEvent(SDL_Event *event);
	bool handleKeyEvent(SDL_Event *event);
private:
	string getStringRepresentation();

	string name_;
	string label_;
	SDLKey thisKey_;
  bool state_;
  bool press_;
	bool grabbingKey_;

	Font *font_;
};

#endif
