//Licence:   LGPL
//Copyright: Visti Andresen









#include "Menu.h"



Menu::Menu(const string &label, const string &name, Font *font, MenuType type)
{
	label_ = label;
	name_ = name;
	font_ = font;
	selectedItem_ = 0;
	menuSpace_ = 2;
	isSelected_ =false;
	hasFocus_ = true;
  screenHeight_ = 768;
  screenWidth_ = 1024;
  type_ = type;
}

Menu::~Menu()
{
	for (unsigned int i=0; i<items_.size(); i++){
		delete items_[i];
	}
}

void Menu::setVirtualScreenSize(float width, float height)
{
  screenWidth_ = width;
  screenHeight_ = height;
}

void Menu::addChild(MenuItem *item)
{
	items_.push_back(item);
}

void Menu::save(ofstream &file)
{
	for (unsigned int i=0; i<items_.size(); i++){
		items_[i]->save(file);
	}	
}

void Menu::load(ifstream &file)
{
	for (unsigned int i=0; i<items_.size(); i++){
		items_[i]->load(file);
	}	
}


void Menu::draw(float x, float y, float zoom)
{
	font_->printString(label_, x, y, zoom);
}

float Menu::itemZoom(unsigned int item)
{
	if (item==selectedItem_) return 2;
  else return 1;
}

void Menu::renderMenu()
{
	isSelected_ = true;
	
	if (hasFocus_ == false){
		if (items_[selectedItem_]->getType()==MENU)
			return ((Menu*)items_[selectedItem_])->renderMenu();
		else hasFocus_ = true;
	}
  
  if (type_ == NORMAL){
    float y = screenHeight_;
    for (unsigned int i=0; i<items_.size(); i++){
     y -= items_[i]->getHeight(menuSpace_);
    }
    y /= 2;
    for (unsigned int i=0; i<items_.size(); i++){
      float x = (screenWidth_-items_[i]->getWidth(itemZoom(i)))/2;
      if (itemZoom(i) == 1){
        items_[i]->draw(x,y,1);
      }
      else {
        items_[i]->draw(x,y-getHeight(itemZoom(i))/4,itemZoom(i));
      }
      y += items_[i]->getHeight(menuSpace_);
    }
  }
  else {
    float yCenter = (screenHeight_/2.0);
      
    //Draw selected item
    float x = (screenWidth_-items_[selectedItem_]->getWidth(
                                              itemZoom(selectedItem_)))/2;
    float yPos = yCenter-(font_->getHeight(itemZoom(selectedItem_))/2.0);
    items_[selectedItem_]->draw(x,yPos,itemZoom(selectedItem_));  
    
    //Draw items below    
    float y = yCenter+items_[selectedItem_]->getHeight(menuSpace_);
    for (unsigned int i=selectedItem_+1; i<items_.size(); i++){
      x = (screenWidth_-items_[i]->getWidth(itemZoom(i)))/2;
      yPos = y-(font_->getHeight(itemZoom(i))/2.0);
      items_[i]->draw(x,yPos,itemZoom(i));
      y += items_[i]->getHeight(menuSpace_);
    }
    
    //Draw items above    
    y = yCenter-items_[selectedItem_]->getHeight(menuSpace_);
    for (unsigned int i=selectedItem_-1; i<items_.size(); i--){
      x = (screenWidth_-items_[i]->getWidth(itemZoom(i)))/2;
      yPos = y-(font_->getHeight(itemZoom(i))/2.0);
      items_[i]->draw(x,yPos,itemZoom(i));
      y -= items_[i]->getHeight(menuSpace_);
    }
  }
}

float Menu::getWidth(float zoom)
{
	return font_->getWidth(label_, zoom);
}

float Menu::getHeight(float zoom)
{
	return font_->getHeight(zoom);
}

bool Menu::handleKeyEvent(SDL_Event *event)
{
	isSelected_ = true;
	
  bool handled=false;
	if (items_[selectedItem_]->getType()!=MENU && items_[selectedItem_]->getType()!=BACK_BUTTON){
		handled = items_[selectedItem_]->handleKeyEvent(event);
	}
	
	if (hasFocus_== false){
		handled = items_[selectedItem_]->handleKeyEvent(event);
		if (!((Menu*)items_[selectedItem_])->getSelected()) hasFocus_ = true;
	}
	
	if (!handled){
		if (event->type != SDL_KEYDOWN) return false;		
		if (event->key.keysym.sym == SDLK_ESCAPE){
			isSelected_ = false;
			return true;
		}
		if (event->key.keysym.sym == SDLK_RETURN){
			if (items_[selectedItem_]->getType()==MENU) {
				hasFocus_ = false;
				return true;
			}
			if (items_[selectedItem_]->getType()==BACK_BUTTON) {
				isSelected_ = false;
				return true;
			}
			else return false;
		}
		if (event->key.keysym.sym == SDLK_DOWN) {
			selectedItem_++;
			if (selectedItem_ >= items_.size()) selectedItem_ = 0;
			return true;
		}
		if (event->key.keysym.sym == SDLK_UP) {
			if (selectedItem_==0) selectedItem_= items_.size()-1;
			else selectedItem_--;
				
			return true;
		}
		return false;
	}
	else return true;
		
	return false;
}

MenuItem *Menu::findItem(const string &name)
{
	MenuItem *result=0;
	for (unsigned int i=0; i<items_.size(); i++){
		if (items_[i]->getName() == name){
			return items_[i];
		}
	}
	for (unsigned int i=0; i<items_.size(); i++){
		if (items_[i]->getType() == MENU){
			result = ((Menu*)items_[i])->findItem(name);
			if (result != 0) return result;
		}
	}
	return result;	
}
