

#ifndef MOD_MENU_H
#define MOD_MENU_H

//Licence:   LGPL
//Copyright: Visti Andresen

#include "MenuItem.h"
#include "TextSelection.h"
#include "TextEntry.h"
#include "Button.h"
#include "BackButton.h"
#include "Key.h"
#include "Font.h"
#include <vector>

/**An class fo handling menus, either */
class Menu : public MenuItem
{
public:
  /**Types defining the apperance of a menu.*/
  enum MenuType {
    NORMAL /**The Menu is centered on screen, the menu must be able to fit
    the screen.*/
    ,SCROLLING /**The Menu moves up and down keeping the "selected" menu item 
    sentered.*/
    };

	Menu(const string &label, const string &name, Font *font, MenuType type = NORMAL);
	virtual ~Menu();
  /**Sets the size of the virtual screen*/
  void setVirtualScreenSize(float width, float height);
  /**Adds a child tho this menu, use this method to add sub menus /
     menu items.*/
	void addChild(MenuItem *item);
  
  /**Loads this menus from the file, will be called recurcivly on the 
      children.*/  
	void load(ifstream &file);
  /**Saves this menus to the file, will be called recurcivly on the 
      children.*/  
	void save(ofstream &file);
	void draw(float x, float y, float zoom);
  
  /**Renders the menu on screen, different from draw in that renderMenu
  renders the menu with children. draw only draws the menu as a sub menu*/
	virtual void renderMenu();
	Type getType(){return MENU;};
	float getWidth(float zoom);
	float getHeight(float zoom);
	string &getName(){return name_;}
	bool handleKeyEvent(SDL_Event *event);
  /**Returns if the current menu is selected, when a menu is selected it should 
    be showed with renderMenu.*/
	bool getSelected(){return isSelected_;}
  
  /**Sets the menus selected state.*/
	void setSelected(bool isSelected){isSelected_ = isSelected;}
  
  /**Recurcivly finds an child og subchild.*/
	MenuItem *findItem(const string &name);

private:
	float itemZoom(unsigned int item);
	
	string label_, name_;
	Font *font_;
	float menuSpace_;
	unsigned int selectedItem_;
	vector<MenuItem*> items_;
	bool hasFocus_;
	bool isSelected_;
  float screenWidth_, screenHeight_;
  MenuType type_;;
};

#endif
