

#ifndef MOD_MENUITEM_H
#define MOD_MENUITEM_H

//Licence:   LGPL
//Copyright: Visti Andresen

#include <SDL.h>
#include <iostream>
#include <fstream>
#include <string>

using namespace std;

/**A class defining everything you can do with a menu item, note that at 
Menu is itself a MenuItem*/
class MenuItem
{
public:
	enum Type {
    MENU           ///A menu or submenu.
    ,CONTROL       ///Some sort of control.
    ,BACK_BUTTON   ///Special control, returns the focus to the parent menu.
  };
  /**Destructor*/
	virtual ~MenuItem(){};
	
  /**Reads the configuration for the MenuItem from any position within the
     file
    
     Hint: Use findLoadPosition to find you entry in the file*/
	virtual void load(ifstream &file){};
	
	/**Saves the MenuItem at the current position in the file.
    
    Entrys must look like "name=value\n".*/
	virtual void save(ofstream &file){};
    
  /**Draws the MenuItem at x,y with a zoom factor of zoom
    (2 equals double size).
    
    The MenuItem may decide not to apply the scale or to only support some zoom
    steps, always check getWidth and getHeight if you need the excact size.
  */
	virtual void draw(float x, float y, float zoom)=0;
    
  /**Returns the Type of this control.*/
	virtual Type getType()=0;
    
  /**Returns the width of this MenuItem at zoom factor zoom, must be excact*/
	virtual float getWidth(float zoom)=0;
  
  /**Returns the height of this MenuItem at zoom factor zoom, must be excact*/
	virtual float getHeight(float zoom)=0;	
    
  /**Returns the name of the MenuItem used when save is called.
    
    The name may not contain a =.*/
	virtual string &getName()=0;
		
  /**Called when a key press/release event occurs, return true if the
    MenuItem has handled the event.

    If true is returned the event handling will stop.*/
	virtual bool handleKeyEvent(SDL_Event *event)=0;
		
protected:
	/**Finds a "name=value" pair in the stream, the stream pointer is left so that
  value can be read directly.
  
  Returns true if the "name=value" pair was found*/
	bool findLoadPosition(const string &name, ifstream &file);
};

#endif
