//Licence:   LGPL
//Copyright: Visti Andresen

#include "FontGl.h"
#include "Menu.h"

//#exe

#include "MenuTest.h"


SDL_Surface *display_;
Menu *menu_;
FontGl *font_;
Button *button1_, *button2_;
Key *key_;
TextEntry *textEntry_;
TextSelection *textSelection_;

int width_=800;
int height_=600;

int main(int argv, char **argc){
  //Prepare SDL and OpenGL
  setupGL();
  
  //Perare a font
  font_ = new FontGl();
  font_->loadFontGfx("FontTestGfx/font.ppm");
  font_->setScreenSize(width_,height_, width_,height_);
  
  //Create a menu
  menu_ = new Menu("Main Menu", "mainmenu", font_);
  menu_->setVirtualScreenSize(width_, height_);
  
  //Add a sub menu
  Menu *subMenu = new Menu("Sub Menu", "submenu", font_);
  menu_->addChild(subMenu);
  subMenu->setVirtualScreenSize(width_, height_);
  button2_ = new Button("Button 2", "button2", font_);
  subMenu->addChild(button2_);
  subMenu->addChild(new BackButton("Back", "", font_));

  //Add a button
  button1_ = new Button("Button 1", "button1", font_);
  menu_->addChild(button1_);

  //Add a Key
  key_ = new Key("Key", "key", font_);
  menu_->addChild(key_);
  
  //Add a TextEntry
  textEntry_ = new TextEntry("Text ", "textentry", font_);
  menu_->addChild(textEntry_);
  
  //Add a TextSelection
  vector<string> choices;
  choices.push_back("Choice 0");
  choices.push_back("Choice 1");
  choices.push_back("Choice 2");
  textSelection_ = new TextSelection("Text Sel. ", "textsel", choices,
                                                                        font_);
  menu_->addChild(textSelection_);                                                                    
  
  //Add a quit button
  menu_->addChild(new BackButton("Exit", "", font_));
  
  bool quit=false;
	SDL_Event event;
  font_->prepareDrawing();
  menu_->setSelected(true);
	menu_->renderMenu();
	while(menu_->getSelected() && !quit){
		glClear(GL_COLOR_BUFFER_BIT);
    font_->prepareDrawing();
		menu_->renderMenu();
		while(SDL_PollEvent(&event)){  
      if (event.type == SDL_QUIT){
        quit = true;
      }
			if (!menu_->handleKeyEvent(&event)){
        key_->handleGameKeyboardEvent(&event);
      }
		}
 		SDL_GL_SwapBuffers();
    
    testMenus();
  }  
  //Destruct
  delete menu_;
  delete font_;
  SDL_Quit();
	return 0;
}

void setupGL()
{
  	if( SDL_Init( SDL_INIT_VIDEO ) < 0 ){
		cout << "Failed initializing SDL Video" << endl;
    exit(-1);
  }

  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
  
  Uint32 flags = SDL_OPENGL|SDL_HWSURFACE;


	display_ = SDL_SetVideoMode(width_, height_, 16, flags);
	if (display_ == 0) {
		SDL_Quit();
		cout << "Could not set video mode" << endl;
    exit(-1);
	}
	
	glViewport(0,0,width_,height_);
  
	/*glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
	gluOrtho2D(0,
                virtualWidth-1,
                virtualHeight-1,
                0);

  */
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
	
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glDisable(GL_DEPTH_TEST);

  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);    
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);

}

void testMenus()
{
  bool press = false;
  if (button1_->wasClicked()){
    press = true;
    cout << "Button 1 was pressed"<<endl;
  }
  if (button2_->wasClicked()){
    press = true;
    cout << "Button 2 was pressed"<<endl;
  }
  if (key_->isPressed()){
    cout << "Keyboard input key is pressed"<<endl;
  }
  
  if (press){
    cout << "Text entry = \"" << textEntry_->getValue()<<"\""<<endl;
    cout << "Text selection = item " << textSelection_->getSelection()<<endl;
    cout << endl;
  }
  
}
