//Licence:   LGPL
//Copyright: Visti Andresen

#include <ctype.h>
#include "TextEntry.h"

TextEntry::TextEntry(const string &label, const string &name,
  Font *font, TextEntryType type, const string &defaultValue, int maxLength)
{
  label_ = label;
	name_ = name;
	value_ = defaultValue;
  type_ = type;
	font_ = font;
  maxLength_ = maxLength;
  grabbingKey_ = false;
}

TextEntry::~TextEntry()
{
}

void TextEntry::load(ifstream &file)
{
  if (findLoadPosition(name_, file)){
    string newValue;
    char c;
    file.read(&c, 1);
    file.read(&c, 1);
		while (file.good() && c!='"'){
      newValue += c;
      file.read(&c, 1);
    }
    value_ = newValue;
	}
}

void TextEntry::save(ofstream &file)
{
  file << name_ << "=\"" << value_ << "\"\n";
}

string TextEntry::getVisualString()
{
  if (grabbingKey_){
    return value_;
  }
  else {
    return label_ + value_;
  }
}

void TextEntry::draw(float x, float y, float zoom)
{
  return font_->printString(getVisualString(), x,y,zoom);
}

float TextEntry::getWidth(float zoom)
{
  return font_->getWidth(getVisualString(), zoom);
}

float TextEntry::getHeight(float zoom)
{
  return font_->getHeight(zoom);
}

bool TextEntry::handleKeyEvent(SDL_Event *event)
{
  if (event->type != SDL_KEYDOWN) return false;	
  SDLKey key = event->key.keysym.sym;
  if (
        key == SDLK_NUMLOCK	||
        key == SDLK_CAPSLOCK	 	||
        key == SDLK_SCROLLOCK	 	||
        key == SDLK_RSHIFT	 	|| 
        key == SDLK_LSHIFT	 	|| 
        key == SDLK_RCTRL	||
        key == SDLK_LCTRL	||
        key == SDLK_RALT	||
        key == SDLK_LALT	||
        key == SDLK_RMETA	||
        key == SDLK_LMETA	||
        key == SDLK_LSUPER	||
        key == SDLK_RSUPER ||
        key == SDLK_MODE){
    return false;
  }
  
  switch (key){
    case SDLK_KP0:
      key = (SDLKey)'0';
      break;
    case SDLK_KP1:
      key = (SDLKey)'1';
      break;
    case SDLK_KP2:
      key = (SDLKey)'2';
      break;
    case SDLK_KP3:
      key = (SDLKey)'3';
      break;
    case SDLK_KP4:
      key = (SDLKey)'4';
      break;
    case SDLK_KP5:
      key = (SDLKey)'5';
      break;
    case SDLK_KP6:
      key = (SDLKey)'6';
      break;
    case SDLK_KP7:
      key = (SDLKey)'7';
      break;
    case SDLK_KP8:
      key = (SDLKey)'8';
      break;
    case SDLK_KP9:
      key = (SDLKey)'9';
      break;
    
    case SDLK_KP_PERIOD:
      key = (SDLKey)'.';
      break;
    case SDLK_KP_DIVIDE:
      key = (SDLKey)'/';
      break;
    case SDLK_KP_MULTIPLY:
      key = (SDLKey)'*';
      break;
    case SDLK_KP_MINUS:
      key = (SDLKey)'-';
      break;
    case SDLK_KP_PLUS:
      key = (SDLKey)'+';
      break;
    case SDLK_KP_EQUALS:
      key = (SDLKey)'=';
      break;
    default:
      break;
  }
  
	if (grabbingKey_){
    if (grabbingKey_ && (key == SDLK_RETURN || key == SDLK_KP_ENTER)){
      grabbingKey_ = false;
      return true;
    }

    bool shiftIsPressed;
    switch (type_) {
      case UPPERCASE:
        shiftIsPressed = true;
        break;
      case LOWERCASE:
        shiftIsPressed = false;
        break;
      default:
        SDLMod mod = event->key.keysym.mod;
        shiftIsPressed = mod & (KMOD_SHIFT);
        break;
    }
    
    char c = (char)(key & 255);
    if (type_ == UPPERCASE){
      c = (char)toupper(c);
    }
    
   
    if (c != '\b'){
      char t[2];
      t[0] = c;
      t[1] = 0;
      string tmp = t;
      if (font_->getWidth(tmp, 1)>0){
        value_ += c;
      }
    }
    else {
      if (value_.size() != 0){
        value_.erase(value_.size()-1 ,1);
      }
    }
    
    return true;
  }
  else if (key == SDLK_RETURN || key == SDLK_KP_ENTER) {
    grabbingKey_ = true;
    return true;   
  }
  else {
    return false;
  }
}
