

#ifndef MOD_TEXTENTRY_H
#define MOD_TEXTENTRY_H

//Licence:   LGPL
//Copyright: Visti Andresen

#include <string>
#include "MenuItem.h"
#include "Font.h"

/**A menu item that allows the user to enter a text string.
  The text value is saved along with the TextEntry when save is called. */
class TextEntry : public MenuItem
{
public:
  /**Defines the type of TextEntry*/
  enum TextEntryType {
    NORMAL      ///Normal TextEntry with both uppercase and lowercase letters.
    ,UPPERCASE  ///Uppercase only TextEntry.
    ,LOWERCASE  ///Lowercase only TextEntry.
    };
  /**Constructor
    
    lable is the name the user can see on screen.

    name is the name of the TextEntry in the config file.

    font is the Font to use when rendering.
    
    type is the type of the TextEntry.
    
    defaultValue is the value to use if no entry is found in the config file 
    or the config file is't loaded.
    
    maxLength is the maximal length of the text the user can enter, 0 for
    unlimited.
    Currently only 0 is implemented.
    */
	TextEntry(const string &label, const string &name, Font *font,
            TextEntryType type = NORMAL, const string &defaultValue="",
            int maxLength_=0);
	~TextEntry();
	void load(ifstream &file);
	void save(ofstream &file);
	void draw(float x, float y, float zoom);
	Type getType(){return CONTROL;}
	float getWidth(float zoom);
	float getHeight(float zoom);
	string &getName(){return name_;}
  /**Returns the text string that the user has typed in the TextEntry.*/
	string &getValue(){return value_;}
	bool handleKeyEvent(SDL_Event *event); 
private:
  string getVisualString();

  string value_;
  TextEntryType type_;
  bool grabbingKey_;
  int maxLength_;

  Font *font_;
	string label_;
	string name_;
};

#endif
