//Licence:   LGPL
//Copyright: Visti Andresen

#include "TextSelection.h"

TextSelection::TextSelection(const string &label, const string &name, vector<string> &choices, Font *font)
{
	label_ = label;
	name_ = name;
	choices_ = choices;
	font_ = font;
	currentSelection_ = 0;
}

TextSelection::~TextSelection()
{
}

void TextSelection::load(ifstream &file)
{
	if (findLoadPosition(name_, file)){
		file >> currentSelection_ ;
	}
}

void TextSelection::save(ofstream &file)
{
	file << name_ << "="<<currentSelection_<<endl;
}

void TextSelection::draw(float x, float y, float zoom)
{
	font_->printString(label_+ choices_[currentSelection_], x, y, zoom);
}

float TextSelection::getWidth(float zoom)
{
	return font_->getWidth(label_+ choices_[currentSelection_], zoom);
}

float TextSelection::getHeight(float zoom)
{
	return font_->getHeight(zoom);
}

bool TextSelection::handleKeyEvent(SDL_Event *event)
{
	if (event->type != SDL_KEYDOWN) return false;
	if (event->key.keysym.sym == SDLK_RIGHT) {
		currentSelection_++;
		if (currentSelection_ >= choices_.size()) currentSelection_ = 0;
		return true;
	}
	if (event->key.keysym.sym == SDLK_LEFT) {
		if (currentSelection_==0) currentSelection_= choices_.size()-1;
		else currentSelection_--;
			
		return true;
	}
	return false;
}
